@@ -5,10 +5,14 @@ package Lemonldap::NG::Portal::Auth::AD;
 
 use strict;
 use Mouse;
-use Lemonldap::NG::Portal::Main::Constants
-  qw(PE_OK PE_PP_PASSWORD_EXPIRED PE_PP_CHANGE_AFTER_RESET);
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_PP_PASSWORD_EXPIRED
+  PE_PP_CHANGE_AFTER_RESET
+  PE_BADCREDENTIALS
+);
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.14';
 
 extends 'Lemonldap::NG::Portal::Auth::LDAP';
 
@@ -66,7 +70,8 @@ sub authenticate {
     my ( $self, $req ) = @_;
     my $res = $self->SUPER::authenticate($req);
 
-    my $pls = $self->ldap->getLdapValue( $req->data->{ldapentry}, 'pwdLastSet' );
+    my $pls =
+      $self->ldap->getLdapValue( $req->data->{ldapentry}, 'pwdLastSet' );
     my $computed = $self->ldap->getLdapValue( $req->data->{ldapentry},
         'msDS-User-Account-Control-Computed' );
     my $_adUac =
@@ -75,6 +80,15 @@ sub authenticate {
 
     unless ( $res == PE_OK ) {
 
+        # Explicit bad credentials message
+        if (    $req->data->{ldapError}
+            and $req->data->{ldapError} =~ /LdapErr: .* data ([^,]+),.*/ )
+        {
+            if ( $1 eq '52e' ) {
+                return PE_BADCREDENTIALS;
+            }
+        }
+
         # Check specific AD attributes
         my $mask = 0xf00000;    # mask to get the 8 at 6th position
         my $expired_flag =
@@ -140,7 +154,7 @@ sub authenticate {
             and $self->conf->{ldapAllowResetExpiredPassword} )
       )
     {
-        $req->data->{oldpassword} = $self->{password};
+        $req->data->{oldpassword} = $req->data->{password};    # Fix 2377
         $req->data->{noerror}     = 1;
         $self->setSecurity($req);
     }
@@ -148,4 +162,16 @@ sub authenticate {
     return $res;
 }
 
+# Define which error codes will stop Combination process
+# @param res error code
+# @return result 1 if stop is needed
+sub stop {
+    my ( $self, $res ) = @_;
+
+    return 1
+      if ( $res == PE_PP_PASSWORD_EXPIRED
+        or $res == PE_PP_CHANGE_AFTER_RESET );
+    return 0;
+}
+
 1;
