@@ -5,6 +5,7 @@ use Mouse;
 use URI;
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Portal::Main::Constants qw(
+  portalConsts
   URIRE
   PE_OK
   PE_INFO
@@ -18,7 +19,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
 );
 use URI;
 
-our $VERSION = '2.18.0';
+our $VERSION = '2.19.0';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::CAS';
@@ -194,15 +195,17 @@ sub run {
         }
 
         my $h = $self->p->processHook( $req, 'casGotRequest', $cas_request );
-        return $h if ( $h != PE_OK );
+        return $self->_failLogin(
+            $req,
+            msg => "casGotRequest hook failed",
+            res => $h
+        ) if ( $h != PE_OK );
 
         my $service = $cas_request->{service};
         $service = '' if ( $self->p->checkXSSAttack( 'service', $service ) );
         my $renew   = $cas_request->{renew};
         my $gateway = $cas_request->{gateway};
 
-        my $casServiceTicket;
-
         # If no service defined, exit
         unless ( defined $service ) {
             $self->logger->debug("No service defined in CAS URL");
@@ -210,8 +213,7 @@ sub run {
         }
 
         unless ( $service =~ m#^(https?://[^/]+)(/.*)?$# ) {
-            $self->logger->error("Bad service $service");
-            return PE_ERROR;
+            return $self->_failAuthorize( $req, msg => "Bad service $service" );
         }
         my $app = $self->getCasApp($service);
 
@@ -245,8 +247,11 @@ sub run {
                 "CAS access control requested on service $service");
 
             unless ($app) {
-                $self->userLogger->error('CAS service not configured');
-                return PE_UNKNOWNPARTNER;
+                return $self->_failLogin(
+                    $req,
+                    msg => 'CAS service not configured',
+                    res => PE_UNKNOWNPARTNER,
+                );
             }
 
             # Check if we have sufficient auth level
@@ -268,103 +273,85 @@ sub run {
                     $self->logger->debug("CAS service $service access allowed");
                 }
                 else {
-                    $self->userLogger->warn( 'User '
-                          . $req->sessionInfo->{ $self->conf->{whatToTrace} }
-                          . " is not authorized to access to $app" );
-
-                    if ( $casAccessControlPolicy =~ /^(error)$/i ) {
-                        $self->logger->debug(
-                                "Return error instead of redirecting user"
-                              . " on CAS service" );
-                        return PE_UNAUTHORIZEDPARTNER;
-                    }
-
-                    else {
-                        $self->logger->debug(
-                            "Redirect user on CAS service with a fake ticket");
-                        $casServiceTicket = "ST-F4K3T1CK3T";
-                    }
+                    return $self->_failLoginRule( $req, $cas_request, $app );
                 }
             }
         }
 
-        if ($app) {
-            $self->userLogger->notice( 'User '
-                  . $req->sessionInfo->{ $self->conf->{whatToTrace} }
-                  . " is authorized to access to $app" );
-        }
-        else {
-            $self->userLogger->notice( 'User '
-                  . $req->sessionInfo->{ $self->conf->{whatToTrace} }
-                  . " is redirected to $service" );
-        }
-
-        unless ($casServiceTicket) {
-
-            # Check last authentication time to decide if
-            # the authentication is recent or not
-            my $casRenewFlag     = 0;
-            my $last_authn_utime = $req->{sessionInfo}->{_lastAuthnUTime} || 0;
-            if (
-                time() - $last_authn_utime <
-                $self->conf->{portalForceAuthnInterval} )
-            {
-                $self->logger->debug(
-                    "Authentication is recent, will set CAS renew flag to true"
-                );
-                $casRenewFlag = 1;
-            }
-
-            # Create a service ticket
+        # Check last authentication time to decide if
+        # the authentication is recent or not
+        my $casRenewFlag     = 0;
+        my $last_authn_utime = $req->{sessionInfo}->{_lastAuthnUTime} || 0;
+        if (
+            time() - $last_authn_utime <
+            $self->conf->{portalForceAuthnInterval} )
+        {
             $self->logger->debug(
-                "Create a CAS service ticket for service $service");
-
-            my $_utime =
-              $self->conf->{casTicketExpiration}
-              ? (
-                time +
-                  $self->conf->{casTicketExpiration} -
-                  $self->conf->{timeout} )
-              : ( $req->{sessionInfo}->{_utime} || time() );
-
-            my $Sinfos;
-            $Sinfos->{type}    = 'casService';
-            $Sinfos->{service} = $service;
-            $Sinfos->{renew}   = $casRenewFlag;
-            $Sinfos->{_cas_id} = $session_id;
-            $Sinfos->{_utime}  = $_utime;
-            $Sinfos->{_casApp} = $app;
-
-            my $h = $self->p->processHook( $req, 'casGenerateServiceTicket',
-                $cas_request, $app, $Sinfos );
-            return $h if ( $h != PE_OK );
-
-            my $casServiceSession = $self->getCasSession( undef, $Sinfos );
-
-            unless ($casServiceSession) {
-                $self->logger->error("Unable to create CAS session");
-                return PE_ERROR;
-            }
-
-            my $casServiceSessionID = $casServiceSession->id;
-            $casServiceTicket = "ST-" . $casServiceSessionID;
-
-            $self->logger->debug(
-                "CAS service session $casServiceSessionID created");
+                "Authentication is recent, will set CAS renew flag to true");
+            $casRenewFlag = 1;
         }
 
-        # Redirect to service
-        # cas_request may have been modified by hook
-        my $service_url = $cas_request->{service};
-        $service_url .= ( $service_url =~ /\?/ ? '&' : '?' )
-          . build_urlencoded( ticket => $casServiceTicket );
+        # Create a service ticket
+        $self->logger->debug(
+            "Create a CAS service ticket for service $service");
 
-        $self->logger->debug("Redirect user to $service_url");
+        my $_utime =
+          $self->conf->{casTicketExpiration}
+          ? (
+            time + $self->conf->{casTicketExpiration} - $self->conf->{timeout} )
+          : ( $req->{sessionInfo}->{_utime} || time() );
 
-        $req->{urldc} = $service_url;
+        my $Sinfos;
+        $Sinfos->{type}    = 'casService';
+        $Sinfos->{service} = $service;
+        $Sinfos->{renew}   = $casRenewFlag;
+        $Sinfos->{_cas_id} = $session_id;
+        $Sinfos->{_utime}  = $_utime;
+        $Sinfos->{_casApp} = $app;
 
-        $req->steps( [] );
-        return PE_OK;
+        $h = $self->p->processHook( $req, 'casGenerateServiceTicket',
+            $cas_request, $app, $Sinfos );
+        return $self->_failLogin(
+            $req,
+            msg => 'casGenerateServiceTicket hook failed',
+            res => $h,
+            ( $app ? ( app => $app ) : () ),
+        ) if ( $h != PE_OK );
+
+        my $casServiceSession = $self->getCasSession( undef, $Sinfos );
+
+        unless ($casServiceSession) {
+            return $self->_failLogin(
+                $req,
+                msg => "Unable to create CAS session",
+                ( $app ? ( app => $app ) : () ),
+            );
+        }
+
+        my $casServiceSessionID = $casServiceSession->id;
+        my $casServiceTicket    = "ST-" . $casServiceSessionID;
+
+        $self->logger->debug(
+            "CAS service session $casServiceSessionID created");
+
+        $self->auditLog(
+            $req,
+            code => "ISSUER_CAS_LOGIN_SUCCESS",
+            ( $app ? ( app => $app ) : () ),
+            service => $service,
+            message => (
+                $app
+                ? ( 'User '
+                      . $req->sessionInfo->{ $self->conf->{whatToTrace} }
+                      . " is authorized to access to $app" )
+                : (     'User '
+                      . $req->sessionInfo->{ $self->conf->{whatToTrace} }
+                      . " is redirected to $service" )
+            ),
+        );
+
+        return $self->_redirectUser( $req, $cas_request->{service},
+            $casServiceTicket );
     }
 
     # 2. LOGOUT
@@ -425,11 +412,7 @@ sub run {
             }
 
             if ($logout_service) {
-                $self->logger->debug(
-                    "User will be redirected to $logout_service");
-                $req->{urldc} = $logout_service;
-                $req->steps( [] );
-                return PE_OK;
+                return $self->_redirectUser( $req, $logout_service );
             }
         }
         else {
@@ -565,8 +548,8 @@ sub validate {
         if (   $service1_uri->rel($service2_uri) eq "./"
             or $service2_uri->rel($service1_uri) eq "./" )
         {
-            $self->logger->notice(
-"Submitted service $service1_uri does not exactly match initial service "
+            $self->logger->notice( "Submitted service $service1_uri"
+                  . " does not exactly match initial service "
                   . $service2_uri
                   . ' but difference is tolerated.' );
         }
@@ -589,9 +572,8 @@ sub validate {
         $self->logger->debug("Renew flag detected ");
 
         unless ( $casServiceSession->data->{renew} ) {
-            $self->logger->error(
-"Authentication renew requested, but not done in former authentication process"
-            );
+            $self->logger->error( "Authentication renew requested,"
+                  . " but not done in former authentication process" );
             $self->deleteCasSession($casServiceSession);
             return $self->returnCasValidateError();
         }
@@ -796,8 +778,8 @@ sub _validate2 {
         if (   $service1_uri->rel($service2_uri) eq "./"
             or $service2_uri->rel($service1_uri) eq "./" )
         {
-            $self->logger->notice(
-"Submitted service $service1_uri does not exactly match initial service "
+            $self->logger->notice( "Submitted service $service1_uri"
+                  . " does not exactly match initial service "
                   . $service2_uri
                   . ' but difference is tolerated.' );
         }
@@ -823,9 +805,8 @@ sub _validate2 {
         $self->logger->debug("Renew flag detected ");
 
         unless ( $casServiceSession->data->{renew} ) {
-            $self->logger->error(
-"Authentication renew requested, but not done in former authentication process"
-            );
+            $self->logger->error( "Authentication renew requested,"
+                  . " but not done in former authentication process" );
             $self->deleteCasSession($casServiceSession);
             return $self->returnCasValidateError();
         }
@@ -876,9 +857,8 @@ sub _validate2 {
 
                 $casProxyGrantingTicketIOU = "PGTIOU-" . $tmpCasSession->id;
                 $self->deleteCasSession($tmpCasSession);
-                $self->logger->debug(
-"Generate proxy granting ticket IOU $casProxyGrantingTicketIOU"
-                );
+                $self->logger->debug( "Generate proxy granting ticket IOU"
+                      . $casProxyGrantingTicketIOU );
 
                 # Request pgtUrl
                 if (
@@ -1023,7 +1003,8 @@ sub _send_back_channel_LogoutRequests {
 
         # Build the URL that could be used to play this logout request
         my $slo_url =
-          $self->p->buildUrl( "cas", "relayLogout", { relay => $relayid } );
+          $self->p->buildUrl( $req->portal, "cas", "relayLogout",
+            { relay => $relayid } );
 
         my $name = $self->getNameFromService($service);
 
@@ -1093,13 +1074,13 @@ sub relayLogout {
 
     if ( $response->is_success ) {
         $self->logger->debug("CAS back-channel logout to $service OK");
-        return $self->p->imgok;
+        return $self->p->imgok($req);
     }
     else {
-        $self->logger->error(
-            "CAS back-channel logout to $service error: " . $response->message );
+        $self->logger->error( "CAS back-channel logout to $service error: "
+              . $response->message );
         $self->logger->debug( $response->dump );
-        return $self->p->imgnok;
+        return $self->p->imgnok($req);
     }
 }
 
@@ -1120,4 +1101,64 @@ sub isLogoutEnabled {
     }
 }
 
+sub _failLogin {
+    my ( $self, $req, %params ) = @_;
+    my $reason = $params{'msg'} ? ": $params{'msg'}" : "";
+    my $res    = $params{res} || PE_ERROR;
+    my $app    = $params{app};
+
+    $self->auditLog(
+        $req,
+        code    => "ISSUER_CAS_LOGIN_FAILED",
+        message => ( "CAS login failed" . $reason ),
+        ( $reason ? ( reason => $reason ) : () ),
+        ( $app    ? ( app    => $app )    : () ),
+        portal_error => portalConsts->{$res},
+    );
+
+    return $res;
+}
+
+sub _failLoginRule {
+    my ( $self, $req, $cas_request, $app ) = @_;
+    my $casAccessControlPolicy = $self->conf->{casAccessControlPolicy};
+
+    $self->auditLog(
+        $req,
+        code => "ISSUER_CAS_LOGIN_FAILED",
+        ( $app ? ( app => $app ) : () ),
+        message => (
+                'User '
+              . $req->sessionInfo->{ $self->conf->{whatToTrace} }
+              . " is not authorized to access to $app"
+        ),
+        reason => "User is not authorized by access rule",
+    );
+
+    if ( $casAccessControlPolicy =~ /^(error)$/i ) {
+        return PE_UNAUTHORIZEDPARTNER;
+    }
+    else {
+        return $self->_redirectUser( $req, $cas_request->{service},
+            "ST-F4K3T1CK3T" );
+    }
+}
+
+sub _redirectUser {
+    my ( $self, $req, $destination, $ticket ) = @_;
+
+    # Redirect to service
+    # cas_request may have been modified by hook
+    if ($ticket) {
+        $destination .= ( $destination =~ /\?/ ? '&' : '?' )
+          . build_urlencoded( ticket => $ticket );
+    }
+
+    $self->logger->debug("Redirect user to $destination");
+
+    $req->{urldc} = $destination;
+    $req->steps( [] );
+    return PE_OK;
+}
+
 1;
