@@ -5,12 +5,10 @@
 # Slave common functions
 package Lemonldap::NG::Portal::Lib::Slave;
 
-use Exporter;
-use base qw(Exporter);
 use strict;
+use Mouse;
 
-our @EXPORT  = qw(checkIP checkHeader);
-our $VERSION = '2.0.6';
+our $VERSION = '2.0.15';
 
 # RUNNING METHODS
 
@@ -35,7 +33,7 @@ sub checkHeader {
       unless ( $self->conf->{slaveHeaderName}
         and $self->conf->{slaveHeaderContent} );
 
-    my $slave_header = 'HTTP_' . uc( $self->{conf}->{slaveHeaderName} );
+    my $slave_header = 'HTTP_' . uc( $self->conf->{slaveHeaderName} );
     $slave_header =~ s/\-/_/g;
     my $headerContent = $req->env->{$slave_header};
     $self->logger->debug(
